#include <libc.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/file.h>
#include <errno.h>

/* extern int	flock(int, int); */

/****************
I want to provide POSIX-style fcntl locking in a non-posix application.
I will do this by replacing posix fcntl calls with the following
fcntl_to_flock, a lame fcntl lock emulator that uses flock

cc -bsd -c fcntl.c

****************/

int fcntl_to_flock(int fd, int cmd, struct flock *arg)
{
  int ret = 0;
  switch (cmd) {
    case F_SETLKW:	/* Blocking Lock */
      if ( (arg->l_type) == F_RDLCK )
        ret=flock(fd, LOCK_SH); 
      else if ( (arg->l_type) == F_WRLCK)
        ret = flock(fd, LOCK_EX); 
      else if (arg->l_type == F_UNLCK )
        ret = flock(fd, LOCK_UN);
      break;

    case F_SETLK:	 /* Nonblocking lock */
      if (arg->l_type == F_RDLCK)
        ret = flock(fd, LOCK_SH | LOCK_NB );
      else if (arg->l_type == F_WRLCK)
        ret = flock(fd, LOCK_EX | LOCK_NB );
      else if ( arg->l_type == F_UNLCK );
        ret = flock(fd, LOCK_UN | LOCK_NB);
      break; 

    case F_GETLK:	/* Get lock status... hmm... */
      if (arg->l_type == F_RDLCK)
      {
        ret = flock(fd, LOCK_SH | LOCK_NB );
        if ( ret == -1 && errno == EWOULDBLOCK)
        { /* We have a winner! Who has the lock?  We don't know, say 0*/
          arg->l_pid = 0; 
        }
        else 
        {  /* Undo the damage */
          flock(fd, LOCK_UN | LOCK_NB );
          arg->l_type = F_UNLCK;
        }
      }
      else if  (arg->l_type == F_WRLCK)
      {
        ret = flock(fd, LOCK_EX | LOCK_NB );
        if (ret == -1 && errno == EWOULDBLOCK)
        {  arg->l_pid = 0; }
        else 
        {  /* Undo the damage */
          flock(fd, LOCK_EX | LOCK_NB );
          arg->l_type = F_UNLCK;
        }
      }

      break;

    default:
	/* Other cmds get passed to the REAL non-posix fcntl */
	ret = fcntl(fd,cmd,(int) arg);
      break;

  } 

  return(ret);  

}
